/*
 * Copyright © 2015, 2016 Red Hat
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice (including the next
 * paragraph) shall be included in all copies or substantial portions of the
 * Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

#ifndef GTK_PRIMARY_SELECTION_SERVER_PROTOCOL_H
#define GTK_PRIMARY_SELECTION_SERVER_PROTOCOL_H

#ifdef  __cplusplus
extern "C" {
#endif

#include <stdint.h>
#include <stddef.h>
#include "wayland-server.h"

struct wl_client;
struct wl_resource;

struct gtk_primary_selection_device;
struct gtk_primary_selection_device_manager;
struct gtk_primary_selection_offer;
struct gtk_primary_selection_source;
struct wl_seat;

extern const struct wl_interface gtk_primary_selection_device_manager_interface;
extern const struct wl_interface gtk_primary_selection_device_interface;
extern const struct wl_interface gtk_primary_selection_offer_interface;
extern const struct wl_interface gtk_primary_selection_source_interface;

/**
 * gtk_primary_selection_device_manager - X primary selection emulation
 * @create_source: create a new primary selection source
 * @get_device: create a new primary selection device
 * @destroy: destroy the primary selection device manager
 *
 * The primary selection device manager is a singleton global object that
 * provides access to the primary selection. It allows to create
 * wp_primary_selection_source objects, as well as retrieving the per-seat
 * wp_primary_selection_device objects.
 */
struct gtk_primary_selection_device_manager_interface {
	/**
	 * create_source - create a new primary selection source
	 * @id: (none)
	 *
	 * Create a new primary selection source.
	 */
	void (*create_source)(struct wl_client *client,
			      struct wl_resource *resource,
			      uint32_t id);
	/**
	 * get_device - create a new primary selection device
	 * @id: (none)
	 * @seat: (none)
	 *
	 * Create a new data device for a given seat.
	 */
	void (*get_device)(struct wl_client *client,
			   struct wl_resource *resource,
			   uint32_t id,
			   struct wl_resource *seat);
	/**
	 * destroy - destroy the primary selection device manager
	 *
	 * Destroy the primary selection device manager.
	 */
	void (*destroy)(struct wl_client *client,
			struct wl_resource *resource);
};


struct gtk_primary_selection_device_interface {
	/**
	 * set_selection - set the primary selection
	 * @source: (none)
	 * @serial: serial of the event that triggered this request
	 *
	 * Replaces the current selection. The previous owner of the
	 * primary selection will receive a
	 * wp_primary_selection_source.cancelled event.
	 *
	 * To unset the selection, set the source to NULL.
	 */
	void (*set_selection)(struct wl_client *client,
			      struct wl_resource *resource,
			      struct wl_resource *source,
			      uint32_t serial);
	/**
	 * destroy - destroy the primary selection device
	 *
	 * Destroy the primary selection device.
	 */
	void (*destroy)(struct wl_client *client,
			struct wl_resource *resource);
};

#define GTK_PRIMARY_SELECTION_DEVICE_DATA_OFFER	0
#define GTK_PRIMARY_SELECTION_DEVICE_SELECTION	1

#define GTK_PRIMARY_SELECTION_DEVICE_DATA_OFFER_SINCE_VERSION	1
#define GTK_PRIMARY_SELECTION_DEVICE_SELECTION_SINCE_VERSION	1

static inline void
gtk_primary_selection_device_send_data_offer(struct wl_resource *resource_, struct wl_resource *offer)
{
	wl_resource_post_event(resource_, GTK_PRIMARY_SELECTION_DEVICE_DATA_OFFER, offer);
}

static inline void
gtk_primary_selection_device_send_selection(struct wl_resource *resource_, struct wl_resource *id)
{
	wl_resource_post_event(resource_, GTK_PRIMARY_SELECTION_DEVICE_SELECTION, id);
}

/**
 * gtk_primary_selection_offer - offer to transfer primary selection
 *	contents
 * @receive: request that the data is transferred
 * @destroy: destroy the primary selection offer
 *
 * A wp_primary_selection_offer represents an offer to transfer the
 * contents of the primary selection clipboard to the client. Similar to
 * wl_data_offer, the offer also describes the mime types that the source
 * will transferthat the data can be converted to and provides the
 * mechanisms for transferring the data directly to the client.
 */
struct gtk_primary_selection_offer_interface {
	/**
	 * receive - request that the data is transferred
	 * @mime_type: (none)
	 * @fd: (none)
	 *
	 * To transfer the contents of the primary selection clipboard,
	 * the client issues this request and indicates the mime type that
	 * it wants to receive. The transfer happens through the passed
	 * file descriptor (typically created with the pipe system call).
	 * The source client writes the data in the mime type
	 * representation requested and then closes the file descriptor.
	 *
	 * The receiving client reads from the read end of the pipe until
	 * EOF and closes its end, at which point the transfer is complete.
	 */
	void (*receive)(struct wl_client *client,
			struct wl_resource *resource,
			const char *mime_type,
			int32_t fd);
	/**
	 * destroy - destroy the primary selection offer
	 *
	 * Destroy the primary selection offer.
	 */
	void (*destroy)(struct wl_client *client,
			struct wl_resource *resource);
};

#define GTK_PRIMARY_SELECTION_OFFER_OFFER	0

#define GTK_PRIMARY_SELECTION_OFFER_OFFER_SINCE_VERSION	1

static inline void
gtk_primary_selection_offer_send_offer(struct wl_resource *resource_, const char *mime_type)
{
	wl_resource_post_event(resource_, GTK_PRIMARY_SELECTION_OFFER_OFFER, mime_type);
}

/**
 * gtk_primary_selection_source - offer to replace the contents of the
 *	primary selection
 * @offer: add an offered mime type
 * @destroy: destroy the primary selection source
 *
 * The source side of a wp_primary_selection_offer, it provides a way to
 * describe the offered data and respond to requests to transfer the
 * requested contents of the primary selection clipboard.
 */
struct gtk_primary_selection_source_interface {
	/**
	 * offer - add an offered mime type
	 * @mime_type: (none)
	 *
	 * This request adds a mime type to the set of mime types
	 * advertised to targets. Can be called several times to offer
	 * multiple types.
	 */
	void (*offer)(struct wl_client *client,
		      struct wl_resource *resource,
		      const char *mime_type);
	/**
	 * destroy - destroy the primary selection source
	 *
	 * Destroy the primary selection source.
	 */
	void (*destroy)(struct wl_client *client,
			struct wl_resource *resource);
};

#define GTK_PRIMARY_SELECTION_SOURCE_SEND	0
#define GTK_PRIMARY_SELECTION_SOURCE_CANCELLED	1

#define GTK_PRIMARY_SELECTION_SOURCE_SEND_SINCE_VERSION	1
#define GTK_PRIMARY_SELECTION_SOURCE_CANCELLED_SINCE_VERSION	1

static inline void
gtk_primary_selection_source_send_send(struct wl_resource *resource_, const char *mime_type, int32_t fd)
{
	wl_resource_post_event(resource_, GTK_PRIMARY_SELECTION_SOURCE_SEND, mime_type, fd);
}

static inline void
gtk_primary_selection_source_send_cancelled(struct wl_resource *resource_)
{
	wl_resource_post_event(resource_, GTK_PRIMARY_SELECTION_SOURCE_CANCELLED);
}

#ifdef  __cplusplus
}
#endif

#endif
